# HAProxy conf workshop

This repository contains all the files required for the HAProxy Conf
2022 "How to Optimize and modernize HAProxy configuration" workshop.

The goal of this repository is to provide attendees all necessary
files to test different setting during and after the workshop.


## Setup and warnings

All the benchmarks were done on Debian 11 hosts with HAProxy version
2.6.6-1~bpo10+1. All the injection traffic was done with
[h1load](https://github.com/wtarreau/h1load).

For the lab setup, 2 servers were used and the hardware was:

* 1 Intel(R) Xeon(R) CPU E5-1630 v3 @ 3.70GHz
* 32GB of RAM
* 1 Intel Corporation 82599ES 10-Gigabit SFI/SFP+ Network Connection

Machines are connected to the same switch and following addressing was
used:

* `198.18.0.101/24` was used for the injector.
* `198.18.0.102/24` was used for HAProxy.


Since the absolute results do not matter and we want to measure a
performance gain, the workshop can also be performed on a
laptop. However some variation can be noticed, mostly regarding
measurement stability. What is important here is to have some orders
of magnitude of configuration impact on performances.

During the demo some hardware optimization were made to obtain stable
results. The most important one is CPU affinity binding. The CPU
topology is shown by `lscpu -e` command:

```
# lscpu -e
CPU NODE SOCKET CORE L1d:L1i:L2:L3 ONLINE    MAXMHZ    MINMHZ
  0    0      0    0 0:0:0:0          yes 3701.0000 1200.0000
  1    0      0    1 1:1:1:0          yes 3701.0000 1200.0000
  2    0      0    2 2:2:2:0          yes 3701.0000 1200.0000
  3    0      0    3 3:3:3:0          yes 3701.0000 1200.0000
  4    0      0    0 0:0:0:0          yes 3701.0000 1200.0000
  5    0      0    1 1:1:1:0          yes 3701.0000 1200.0000
  6    0      0    2 2:2:2:0          yes 3701.0000 1200.0000
  7    0      0    3 3:3:3:0          yes 3701.0000 1200.0000
```

* `haproxy` is arbitrary bond to CPU 0 (check `cpu-map 1/all 0` in
  the configuration files).
* `h1load` is arbitrary bond to CPU 1 or CPU 0 to 3 depending on
  benchmarks. CPU binding is done using the `taskset` command.
* To provide all CPU cycles to both `haproxy` and `h1load`, network
  device interrupts are bound to CPU 7.  Only one core was required
  because the `system interrupt` percentage never reached 100% in
  `top`. The network interface bing to CPU cores selection goes beyond
  this workshop. The following command is used:

```
for i in $(grep eth2 /proc/interrupts | cut -f 1 -d:); do
    echo 80 > /proc/irq/$i/smp_affinity
done
```

Only the source IP deny workshop required to use 2 cores to bind the
network interface to.


For further details about `smp_affinity` please consult the [kernel
documentation](https://www.kernel.org/doc/html/latest/core-api/irq/irq-affinity.html)

To figure out how many connection are required, HAProxy should be
saturated (take 100% of its CPU core). The optimal number of clients
(concurrent connection) should be a power of 2 and spread evenly over
the injector cores. For most tests 64 was a good value and for some
tests, such as connection close, 128 was a better value.

## Practical workshops

Each practical workshop has is own directory and its own `README` file
with all instructions about how to run the tests.

* `01-acl-lookup`
* `02-closing-connections`
* `03-backend-selection`
* `04-stick-tables`
* `05-misc`

In addition all benchmark results files are provided in `results`
directory for all workshops.
